########################### PSQF 6270 Example 3 using R #################################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sorting

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("prediction")) install.packages("prediction")
library(prediction) # To get predicted values like Stata does

#if (!require("VGAM")) install.packages("VGAM")
#library(VGAM) # To fit lots of count models with difficulty (not used here)

if (!require("MASS")) install.packages("MASS")
library(MASS) # To fit negative binomial model easily

if (!require("pscl")) install.packages("pscl")
library(pscl) # To fit zero-inflated and hurdle count models easily


######################################################################################
#####                  BEGIN DATA MANIPULATION OF EXAMPLE 3                      #####
#####                   CHANGE "filesave" TO YOUR FOLDER PATH                    #####
######################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/23_PSQF6270/PSQF6270_Example3/"
filename = "Excel_Example3.xlsx"
setwd(dir=filesave)

# Import Example 3 Excel data
Example3 = read_excel(paste0(filesave,filename))
# Convert to data frame without labels to use for analysis
Example3 = as.data.frame(Example3)

# R code to create indicator-dummy-coded binary predictors
Example3$NvC=NA; Example3$NvS=NA  # Make 2 new empty variables 
Example3$NvC[which(Example3$ercond==1)]=0  # Replace if ercond=None=1
Example3$NvS[which(Example3$ercond==1)]=0
Example3$NvC[which(Example3$ercond==2)]=1  # Replace if ercond=CogR=2
Example3$NvS[which(Example3$ercond==2)]=0
Example3$NvC[which(Example3$ercond==3)]=0  # Replace if ercond=Supp=3
Example3$NvS[which(Example3$ercond==3)]=1
# Label variables as comments only (not actually added to data)
#ipv=    "ipv: Inter-Partner Violence (0=N,1=Y)"
#ercond= "ercond: 1=None, 2=CogR, 3=Supp" 
#aggr=   "aggr: Aggressive Verbalizations"
#NvC=    "NvC: Condition None=0 vs CogR=1"
#NvS=    "NvS: Condition None=0 vs Supp=1"

# Filter to only cases complete on all variables to be used below
Example3 = Example3[complete.cases(Example3[ , c("aggr","IPV","ercond")]),]


######################################################################################
#####                  BEGIN EXAMPLE 3 COUNT OUTCOME MODELS                      #####
######################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example3_R_Output.txt"))

print("R Cell Means for Aggressive Verbalizations Outcome")
describeBy(x=Example3$aggr,list(Example3$IPV,Example3$ercond))

# to save a plot: open a file, create the plot, then close the file
png(file = "R Overall Histogram.png")  # open file
hist(x=Example3$aggr, freq=FALSE,
     ylab="Density",xlab="aggr: Aggressive Verbalizations") # axis labels
dev.off()  # close file
# I did not figure out how to make a separate histogram for each cell

txtComment(" ") # insert blank space in output 


######################################################################################
print("R Link=Identity Dist=Normal Model using glm for ML Estimation")
print("Also using t instead of z for univariate Wald tests")
ModelNorm = glm(data=Example3, family=gaussian(link="identity"), 
                formula=aggr~1+IPV+NvC+NvS +IPV:NvC +IPV:NvS)
print("Print results with -2LL")
summary(ModelNorm); -2*logLik(ModelNorm) 

print("DF=5 Multiv Wald Test of Model with 8 digits")
NormR2 = glht(model=ModelNorm, 
              linfct=c("IPV=0","NvC=0","NvS=0","IPV:NvC=0","IPV:NvS=0"))
print(summary(NormR2, test=Chisqtest()), digits="8") # Joint chi-square test 

print("DF=2 Multiv Wald Test of Interaction with 8 digits")
NormInt = glht(model=ModelNorm, linfct=c("IPV:NvC=0","IPV:NvS=0"))
print(summary(NormInt, test=Chisqtest()), digits="8") # Joint chi-square test 

print("Yhat cell means in original count model scale per condition") 
NormPredN = prediction(model=ModelNorm, type="response",
                       at=list(IPV=0:1,NvC=0,NvS=0))
NormPredC = prediction(model=ModelNorm, type="response",
                       at=list(IPV=0:1,NvC=1,NvS=0))
NormPredS = prediction(model=ModelNorm, type="response",
                       at=list(IPV=0:1,NvC=0,NvS=1))
summary(rbind(NormPredN,NormPredC,NormPredS))

print("Simple slopes: condition per IPV, IPV per condition, interactions")
NormSlopes = (summary(glht(model=ModelNorm, linfct=rbind(
  "No vs Yes IPV: None"   = c(0,1, 0,0, 0,0),   # in order of fixed effects
  "No vs Yes IPV: CogR"   = c(0,1, 0,0, 1,0),
  "No vs Yes IPV: Supp"   = c(0,1, 0,0, 0,1),
  "None vs CogR: IPV=No" =  c(0,0, 1,0, 0,0), 
  "None vs Supp: IPV=No" =  c(0,0, 0,1, 0,0),
  "CogR vs Supp: IPV=No" =  c(0,0,-1,1, 0,0),
  "None vs CogR: IPV=Yes" = c(0,0, 1,0, 1,0),  
  "None vs Supp: IPV=Yes" = c(0,0, 0,1, 0,1),
  "CogR vs Supp: IPV=Yes" = c(0,0,-1,1,-1,1),
  "No/Yes IPV differ btw None/CogR" = c(0,0,0,0, 1,0),  
  "No/Yes IPV differ btw None/Supp" = c(0,0,0,0, 0,1),
  "No/Yes IPV differ btw CogR/Supp" = c(0,0,0,0,-1,1))),test=adjusted("none")))
NormSlopes

txtComment(" ") # insert blank space in output 


######################################################################################
# R: save sample size and DDF for conditional distribution fit
DDFn=225     # What SAS GLIMMIX uses by default
DDFk=DDFn-6  # What STATA and SAS GENMOD use by default (N - # fixed effects)
######################################################################################

print("R Link=Log Dist=Poisson Model using glm")
ModelPoisson = glm(data=Example3, family=poisson(link="log"),
                   formula=aggr~1+IPV+NvC+NvS +IPV:NvC +IPV:NvS)
print("Print results with -2LL")
summary(ModelPoisson); -2*logLik(ModelPoisson)
print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelPoisson, type="pearson")^2)/DDFk  # STATA version

print("DF=5 Multiv Wald Test of Model with 8 digits")
PoissonR2 = glht(model=ModelPoisson, 
                 linfct=c("IPV=0","NvC=0","NvS=0","IPV:NvC=0","IPV:NvS=0"))
print(summary(PoissonR2, test=Chisqtest()), digits="8") # Joint chi-square test 

print("DF=2 Multiv Wald Test of Interaction with 8 digits")
PoissonInt = glht(model=ModelPoisson, linfct=c("IPV:NvC=0","IPV:NvS=0"))
print(summary(PoissonInt, test=Chisqtest()), digits="8") # Joint chi-square test 

print("Yhat cell means in log count model scale per condition") 
PoissonLogN = prediction(model=ModelPoisson, type="link",
                         at=list(IPV=0:1,NvC=0,NvS=0))
PoissonLogC = prediction(model=ModelPoisson, type="link",
                         at=list(IPV=0:1,NvC=1,NvS=0))
PoissonLogS = prediction(model=ModelPoisson, type="link",
                         at=list(IPV=0:1,NvC=0,NvS=1))
summary(rbind(PoissonLogN,PoissonLogC,PoissonLogS))

print("Yhat cell means in expected count data scale per condition") 
PoissonCountN = prediction(model=ModelPoisson, type="response",
                          at=list(IPV=0:1,NvC=0,NvS=0))
PoissonCountC = prediction(model=ModelPoisson, type="response",
                           at=list(IPV=0:1,NvC=1,NvS=0))
PoissonCountS = prediction(model=ModelPoisson, type="response",
                           at=list(IPV=0:1,NvC=0,NvS=1))
summary(rbind(PoissonCountN,PoissonCountC,PoissonCountS))

print("Simple slopes: condition per IPV, IPV per condition, interactions")
PoissonSlopes = (summary(glht(model=ModelPoisson, linfct=rbind(
  "No vs Yes IPV: None"   = c(0,1, 0,0, 0,0),   # in order of fixed effects
  "No vs Yes IPV: CogR"   = c(0,1, 0,0, 1,0),
  "No vs Yes IPV: Supp"   = c(0,1, 0,0, 0,1),
  "None vs CogR: IPV=No" =  c(0,0, 1,0, 0,0), 
  "None vs Supp: IPV=No" =  c(0,0, 0,1, 0,0),
  "CogR vs Supp: IPV=No" =  c(0,0,-1,1, 0,0),
  "None vs CogR: IPV=Yes" = c(0,0, 1,0, 1,0),  
  "None vs Supp: IPV=Yes" = c(0,0, 0,1, 0,1),
  "CogR vs Supp: IPV=Yes" = c(0,0,-1,1,-1,1),
  "No/Yes IPV differ btw None/CogR" = c(0,0,0,0, 1,0),  
  "No/Yes IPV differ btw None/Supp" = c(0,0,0,0, 0,1),
  "No/Yes IPV differ btw CogR/Supp" = c(0,0,0,0,-1,1))),test=adjusted("none")))
PoissonSlopes; print("IRR effect sizes for simple slopes")
data.frame(OR=exp(PoissonSlopes$test$coefficients))

txtComment(" ") # insert blank space in output


######################################################################################
print("R Link=Log Dist=Negative Binomial Model")
print("Using glm.nb add-on to glm from MASS package")
ModelNegBin = glm.nb(data=Example3, link=log,
                     formula=aggr~1+IPV+NvC+NvS +IPV:NvC +IPV:NvS)
print("Print results with -2LL")
summary(ModelNegBin); -2*logLik(ModelNegBin)
print("Scale factor in same k metric as SAS and STATA")
1/ModelNegBin$theta
print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelNegBin, type="pearson")^2)/DDFk  # STATA version

print("Likelihood Ratio Test for Poisson vs NegBin")
DevTest=-2*(logLik(ModelPoisson)-logLik(ModelNegBin))
RegPvalue=pchisq((DevTest), df=1, lower.tail=FALSE); MixPvalue=RegPvalue/2
print("Test Statistic, Regular and Mixture P-values for DF=1") 
DevTest; RegPvalue; MixPvalue

print("DF=5 Multiv Wald Test of Model with 8 digits")
NegBinR2 = glht(model=ModelNegBin, 
                linfct=c("IPV=0","NvC=0","NvS=0","IPV:NvC=0","IPV:NvS=0"))
print(summary(NegBinR2, test=Chisqtest()), digits="8") # Joint chi-square test 

print("DF=2 Multiv Wald Test of Interaction with 8 digits")
NegBinInt = glht(model=ModelNegBin, linfct=c("IPV:NvC=0","IPV:NvS=0"))
print(summary(NegBinInt, test=Chisqtest()), digits="8") # Joint chi-square test 

print("Yhat cell means in log count model scale per condition") 
NegBinLogN = prediction(model=ModelNegBin, type="link",
                         at=list(IPV=0:1,NvC=0,NvS=0))
NegBinLogC = prediction(model=ModelNegBin, type="link",
                         at=list(IPV=0:1,NvC=1,NvS=0))
NegBinLogS = prediction(model=ModelNegBin, type="link",
                         at=list(IPV=0:1,NvC=0,NvS=1))
summary(rbind(NegBinLogN,NegBinLogC,NegBinLogS))

print("Yhat cell means in expected count data scale per condition") 
NegBinCountN = prediction(model=ModelNegBin, type="response",
                          at=list(IPV=0:1,NvC=0,NvS=0))
NegBinCountC = prediction(model=ModelNegBin, type="response",
                          at=list(IPV=0:1,NvC=1,NvS=0))
NegBinCountS = prediction(model=ModelNegBin, type="response",
                          at=list(IPV=0:1,NvC=0,NvS=1))
summary(rbind(NegBinCountN,NegBinCountC,NegBinCountS))

print("Simple slopes: condition per IPV, IPV per condition, interactions")
NegBinSlopes = (summary(glht(model=ModelNegBin, linfct=rbind(
  "No vs Yes IPV: None"   = c(0,1, 0,0, 0,0),   # in order of fixed effects
  "No vs Yes IPV: CogR"   = c(0,1, 0,0, 1,0),
  "No vs Yes IPV: Supp"   = c(0,1, 0,0, 0,1),
  "None vs CogR: IPV=No" =  c(0,0, 1,0, 0,0), 
  "None vs Supp: IPV=No" =  c(0,0, 0,1, 0,0),
  "CogR vs Supp: IPV=No" =  c(0,0,-1,1, 0,0),
  "None vs CogR: IPV=Yes" = c(0,0, 1,0, 1,0),  
  "None vs Supp: IPV=Yes" = c(0,0, 0,1, 0,1),
  "CogR vs Supp: IPV=Yes" = c(0,0,-1,1,-1,1),
  "No/Yes IPV differ btw None/CogR" = c(0,0,0,0, 1,0),  
  "No/Yes IPV differ btw None/Supp" = c(0,0,0,0, 0,1),
  "No/Yes IPV differ btw CogR/Supp" = c(0,0,0,0,-1,1))),test=adjusted("none")))
NegBinSlopes; print("IRR effect sizes for simple slopes")
data.frame(OR=exp(NegBinSlopes$test$coefficients))

print("Save predicted counts and correlate with aggr") 
Example3$PredCount = predict(ModelNegBin, type="response")
rPred = cor.test(Example3$PredCount, Example3$aggr, method="pearson")
print("R2"); rPred$estimate^2

txtComment(" ") # insert blank space in output


######################################################################################
print("R Link=Log Dist=Zero-Inflated Poisson Model using pscl package")
print("Only intercept in zero-inflation model, predict logit of extra zero")
ModelZIP = zeroinfl(data=Example3, dist="poisson",link="logit",
                    formula=aggr~1+IPV+NvC+NvS +IPV:NvC +IPV:NvS | 1)
print("Print results with -2LL")
summary(ModelZIP); -2*logLik(ModelZIP)
print("Pearson Chi-Square / DF Fit")
sum(residuals(ModelZIP, type="pearson")^2)/(DDFk-1) # Not given in STATA
print("Get probability of being an extra 0")
ZIPprob=1/(1+exp(-1*ModelZIP$coefficients$zero)); ZIPprob 
# glht apparently does not work with pscl package, so no Wald tests included here

txtComment(" ") # insert blank space in output


######################################################################################
print("R Link=Log Dist=Zero-Inflated Negative Binomial Model using pscl package")
print("Only intercept in zero-inflation model, predict logit of extra zero")
ModelZINB = zeroinfl(data=Example3, dist="negbin",link="logit",
                     formula=aggr~1+IPV+NvC+NvS +IPV:NvC +IPV:NvS | 1)
print("Print results with -2LL")
summary(ModelZINB); -2*logLik(ModelZINB)
print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelZINB, type="pearson")^2)/(DDFk-1) # Not given in STATA
print("Scale factor in same k metric as SAS and STATA")
1/ModelZINB$theta
print("Get probability of being an extra 0")
ZINBprob=1/(1+exp(-1*ModelZINB$coefficients$zero)); ZINBprob 
# glht apparently does not work with pscl package, so no Wald tests included here

print("Likelihood Ratio Test for NegBin vs ZINB")
DevTest=-2*(logLik(ModelNegBin)-logLik(ModelZINB))
RegPvalue=pchisq((DevTest), df=1, lower.tail=FALSE); MixPvalue=RegPvalue/2
print("Test Statistic, Regular and Mixture P-values for DF=1") 
DevTest; RegPvalue; MixPvalue


txtComment(" ") # insert blank space in output


# I could not find heterogeneous negative binomial readily available in R

######################################################################################
# Close external output file
txtStop()




